@@ -8,11 +8,12 @@ class JSONMixin:
     def to_dict(self) -> dict: ...
 
     @classmethod
-    def from_dict(cls, o: dict): ...
+    def from_dict(cls, o: dict, *, infer_missing=True): ...
 
     def to_json(self, *, separators=None, indent=None) -> str: ...
 
-    def from_json(cls, json_data: Union[str, bytes]): ...
+    def from_json(cls, json_data: Union[str, bytes],
+                  *, infer_missing=True): ...
 
 
 def dataclass_json(cls=None):
